'''
The MIT License (MIT)

https://github.com/robertchase/spindrift/blob/master/LICENSE.txt
'''
import importlib


def import_by_path(target):
    if isinstance(target, str):
        modnam, funnam = target.rsplit('.', 1)
        mod = importlib.import_module(modnam)
        return getattr(mod, funnam)
    return target



'''
The MIT License (MIT)

https://github.com/robertchase/ergaleia/blob/master/LICENSE.txt
'''
from ergaleia.normalize_path import normalize_path


def load_from_path(path, filetype=None, has_filetype=True):
    """ load file content from a file specified as dot-separated

        The file is located according to logic in normalize_path,
        and the contents are returned. (See Note 1)

        Parameters: (see normalize_path)
            path         - dot-separated path
            filetype     - optional filetype
            has_filetype - if True, treat last dot-delimited token as filetype

        Notes:
            1. If path is a file-like object, then data is read directly
               from path, without trying to open it.
            2. Non-string paths are returned immediately (excluding the
               case in Note 1).
            3. If has_filetype is True, filetype does not have to be specified.
               If filetype is specified, has_filetype is ignored, and filetype
               must match the last dot-delimited token exactly.
    """
    if not isinstance(path, str):
        try:
            return path.read()
        except AttributeError:
            return path
    path = normalize_path(path, filetype, has_filetype)
    with open(path) as data:
        return data.read()


def load_lines_from_path(path, filetype=None, has_filetype=True):
    """ load lines from a file specified as dot-separated

        The file is located according to logic in normalize_path,
        and a list of lines is returned. (See Note 1)

        Parameters: (see normalize_path)
            path         - dot-separated path
            filetype     - optional filetype
            has_filetype - if True, treat last dot-delimited token as filetype

        Notes:
            1. If path is a file-like object, then lines are read directly
               from path, without trying to open it.
            2. Non-string paths are returned immediately (excluding the
               case in Note 1).
            3. If has_filetype is True, filetype does not have to be specified.
               If filetype is specified, has_filetype is ignored, and filetype
               must match the last dot-delimited token exactly.
    """
    if not isinstance(path, str):
        try:
            return path.readlines()
        except AttributeError:
            return path
    path = normalize_path(path, filetype)
    with open(path) as data:
        return data.readlines()



'''
The MIT License (MIT)

https://github.com/robertchase/ergaleia/blob/master/LICENSE.txt
'''


def nested_get(d, keys, default=None, required=False, as_list=False):
    """ Multi-level dict get helper

        Parameters:

            d        - dict instance
            keys     - iterable of keys or dot-delimited str of keys (see
                       Note 1)
            default  - value if index fails
            required - require every index to work (see Note 2)
            as_list  - return result as list (see Note 3)

        Notes:
            1. Each key is used to index a dict, replacing the dict with
               the matching value. This process is repeated until an index
               fails, or the keys are exhausted.

               If keys is a string, it is split by '.' and treated as a
               list of keys.
            2. If the required flag is False, a failure to match a key
               causes the default value to be used. If the required flag is
               True, every key must match, otherwise a TypeError or KeyError is
               raised.
            3. If as_list is True, a non-list match will be wrapped in a list,
               unless match is None, which will be replaced with an empty list.
    """
    if isinstance(keys, str):
        keys = keys.split('.')
    for key in keys:
        try:
            d = d[key]
        except KeyError:
            if required:
                raise
            d = default
            break
        except TypeError:
            if required:
                raise
            d = default
            break
    if as_list:
        return [] if d is None else [d] if not isinstance(d, list) else d
    return d



'''
The MIT License (MIT)

https://github.com/robertchase/ergaleia/blob/master/LICENSE.txt
'''
import sys
from xml.sax import make_parser
from xml.sax.handler import ContentHandler

PY3 = sys.version_info.major == 3

if PY3:
    from io import StringIO
else:
    from StringIO import StringIO


class XmlToDict(ContentHandler):

    def __init__(self, groupby=None):
        self.data = ''
        self.stack = [(None, {})]

    def startElement(self, name, attrs):
        self.stack.append((name, {n: v for n, v in attrs.items()}))

    def endElement(self, name):
        value = self.data.strip()
        self.data = ''

        name, collection = self.stack.pop()
        p_name, p_collection = self.stack[-1]

        if not value:
            value = collection

        if name in p_collection:
            p_value = p_collection[name]
            if not isinstance(p_value, (list, tuple)):
                p_value = p_collection[name] = [p_value]
            p_value.append(value)
        else:
            p_collection[name] = value

    def characters(self, ch):
        self.data += ch if PY3 else ch.encode('ascii')


def from_xml(data, handler_class=XmlToDict):
    if isinstance(data, str):
        data = StringIO(data)
    handler = handler_class()
    p = make_parser()
    p.setContentHandler(handler)
    p.parse(data)
    return handler.stack[0][1]



from setuptools import find_packages
from distutils.core import setup

setup(
    name='ergaleia',
    version='1.2.0rc1',
    packages=find_packages(exclude=['tests']),
    description='A library of random tools',
    long_description="""
Documentation
-------------
    You can see the project and documentation at the `GitHub repo <https://github.com/robertchase/ergaleia>`_
    """,
    author='Bob Chase',
    url='https://github.com/robertchase/ergaleia',
    license='MIT',
)
